package Addressbook;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: Action.java,v 1.1.1.1 2001/12/06 19:33:52 bradford Exp $
 */

import org.xmldb.api.DatabaseManager;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Vector;

import org.xmldb.api.base.*;
import org.xmldb.api.modules.*;

import org.w3c.dom.*;
import org.apache.xpath.*;
import org.apache.xpath.objects.*;

import javax.servlet.*;
import javax.servlet.http.*;

/**
 * Common class for the Addressbook
 */
public abstract class Action extends HttpServlet {

   // Version of the XMLDB Api that we are using
   public final static String XMLDBAPIVERSION = "1.0";

   // Singleton class used to hold connnection to database
   protected static DBConnection connection = new DBConnection();

   /**
    * Return a collection object for the addressbook
    */
   public Collection getCollection(HttpServletRequest request, HttpServletResponse response) throws javax.servlet.ServletException, java.io.IOException {
      return connection.getCollectionInstance(request, response);
   }

   /**
    * Return a vector of hashtables populated with all elements in
    * ResourceIterator
    */
   public Vector getContactVectorFromIterator(ResourceIterator results) {

      // new Vector instance to be returned
      Vector vector = new Vector();
      try {
         while (results.hasMoreResources()) {

            // Create an XObject to be used in Xpath query
            XObject xo = new XObject();
            // Create a new hastable to store into vector
            Hashtable table = new Hashtable();

            // Retrieve the next node
            XMLResource resource = (XMLResource) results.nextResource();
            Node originalnode = resource.getContentAsDOM();

            // Get the document key for this node for later use
            String dockey = resource.getDocumentId();
            table.put("dockey", dockey);

            // Use Xalan xpath parser to extract data, store into hastable
            xo = XPathAPI.eval(originalnode, "/person/fname");
            table.put("fname", xo.toString());

            xo = XPathAPI.eval(originalnode, "/person/lname");
            table.put("lname", xo.toString());

            xo = XPathAPI.eval(originalnode, "/person/phone[@type='work']");
            table.put("workphone", xo.toString());

            xo = XPathAPI.eval(originalnode, "/person/phone[@type='home']");
            table.put("homephone", xo.toString());

            xo = XPathAPI.eval(originalnode, "/person/phone[@type='cell']");
            table.put("cellphone", xo.toString());

            xo = XPathAPI.eval(originalnode, "/person/email[@type='home']");
            table.put("homeemail", xo.toString());

            xo = XPathAPI.eval(originalnode, "/person/email[@type='work']");
            table.put("workemail", xo.toString());

            xo = XPathAPI.eval(originalnode, "/person/address[@type='home']");
            table.put("homeaddress", xo.toString());

            xo = XPathAPI.eval(originalnode, "/person/address[@type='work']");
            table.put("workaddress", xo.toString());

            // Add hashtable to vector to pass on to JSP page
            vector.add(table);
         }

      }
      catch (Exception e) {
         e.printStackTrace();
      }

      return vector;
   }

   /**
    * Create a String XML document from passed in parameters
    */
   public String toXml(String fname, String lname, String workphone,
         String homephone, String cellphone, String homeemail, String workemail,
         String homeaddress, String workaddress) {

      String ourdoc;

      // Build up the string to represent our document
      ourdoc = "<person>";
      ourdoc = ourdoc + "<fname>" + fname + "</fname>";
      ourdoc = ourdoc + "<lname>" + lname + "</lname>";
      ourdoc = ourdoc + "<phone type='work'>" + workphone + "</phone>";
      ourdoc = ourdoc + "<phone type='home'>" + homephone + "</phone>";
      ourdoc = ourdoc + "<phone type='cell'>" + cellphone + "</phone>";
      ourdoc = ourdoc + "<email type='home'>" + homeemail + "</email>";
      ourdoc = ourdoc + "<email type='work'>" + workemail + "</email>";
      ourdoc = ourdoc + "<address type='home'>" + homeaddress + "</address>";
      ourdoc = ourdoc + "<address type='work'>" + workaddress + "</address>";
      ourdoc = ourdoc + "</person>";

      // Return the created XML string
      return ourdoc;
   }

}

