package org.apache.xindice.client.xmldb.services;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: XUpdateQueryServiceImpl.java,v 1.2 2002/01/16 09:48:39 kstaken Exp $
 */

import org.apache.xindice.core.FaultCodes;
import org.apache.xindice.client.corba.db.*;
import org.apache.xindice.client.corba.*;
import org.apache.xindice.client.corba.db.Collection;
import org.apache.xindice.client.xmldb.*;

import org.apache.xindice.client.xmldb.resources.*;

import org.apache.xindice.xml.*;
import org.apache.xindice.xml.dom.*;

import org.w3c.dom.*;
import org.xmldb.api.base.*;
import org.xmldb.api.modules.*;

import java.util.*;

public class XUpdateQueryServiceImpl extends CommonConfigurable
   implements org.xmldb.api.modules.XUpdateQueryService
{
   protected org.xmldb.api.base.Collection collection = null;
   private NamespaceMap nsMap = new NamespaceMap();

   public XUpdateQueryServiceImpl() {
   }

   public String getName() throws XMLDBException {
      return "XUpdateQueryService";
   }

   public String getVersion() throws XMLDBException {
      return "1.0";
   }

   public void setCollection(org.xmldb.api.base.Collection col)
         throws XMLDBException {
      this.collection = col;
   }


    public void setDefaultNamespace(String uri) throws XMLDBException {
       if ( uri != null )
          nsMap.setDefaultNamespace(uri);
       else
          nsMap.removeDefaultNamespace();
    }

    public void removeDefaultNamespace() {
       nsMap.removeDefaultNamespace();
    }

    public void setNamespace(String prefix, String uri) throws XMLDBException {
       if ( ( prefix == null ) || prefix.equals("") ) {
          setDefaultNamespace(uri);
       }
       else {
          if ( uri != null ) {
             nsMap.setNamespace(prefix, uri);
          }
       }
    }

    public void removeNamespace(String prefix) {
       if ( ( prefix == null ) || prefix.equals("") ) {
          removeDefaultNamespace();
       }

       nsMap.removeNamespace(prefix);
    }

    public String getDefaultNamespace() {
       return nsMap.getDefaultNamespaceURI();
    }

    public String getNamespace(String prefix) {
       if ( ( prefix == null ) || prefix.equals("") ) {
          return nsMap.getDefaultNamespaceURI();
       }

       return nsMap.getNamespaceURI(prefix);
    }

    public void clearNamespaces() {
       nsMap.clear();
    }

    private NamedVal[] getNamespaces() {
       NamedVal[] nv = new NamedVal[nsMap.size()];
       Iterator iter = nsMap.keySet().iterator();
       int i = 0;
       while ( iter.hasNext() ) {
          String prefix = (String)iter.next();
          String uri = nsMap.getNamespaceURI(prefix);
          nv[i++] = new NamedVal(prefix, uri);
       }
       return nv;
    }

   public XMLResource updateResult(String commands) throws XMLDBException {
      // Get the Xindice collection object from the XML:DB collection wrapper.
      Collection col = ((CollectionImpl) collection).getServerObject();

      try {
         EncodedBuffer buffer = col.queryCollection("XUpdate", commands, getNamespaces(), -1);
         return new XMLResourceImpl("", collection, new String(buffer.buf));
      }
      catch (Exception e) {
      e.printStackTrace();
         throw FaultCodes.createXMLDBException(e);
      }
   }

   /**
    * Runs a set of XUpdate operations against the collection. All selected
    * documents are to be updated and stored back to the repository.
    *
    * @param commands The XUpdate commands to use.
    * @return the number of nodes updated
    * @exception XMLDBException with expected error codes.<br />
    *  <code>ErrorCodes.VENDOR_ERROR</code> for any vendor
    *  specific errors that occur.<br />
    */
   public long update(String commands) throws XMLDBException {
      XMLResource result = updateResult(commands);
      return getResultCount(result);
   }

   public XMLResource updateResourceResult(String id, String commands) throws XMLDBException {
      // Get the Xindice collection object from the XML:DB collection wrapper.
      Collection col = ((CollectionImpl) collection).getServerObject();

      try {
         EncodedBuffer buffer = col.queryDocument("XUpdate", commands, getNamespaces(), id, -1);
         return new XMLResourceImpl("", collection, new String(buffer.buf));
      }
      catch (Exception e) {      
         throw FaultCodes.createXMLDBException(e);
      }
   }

   /**
    * Runs a set of XUpdate operations against a resource stored in a
    * collection. The resource will be updated in place in the collection.
    *
    * @param commands The XUpdate commands to use.
    * @return the number of nodes updateds
    * @exception XMLDBException with expected error codes.<br />
    *  <code>ErrorCodes.VENDOR_ERROR</code> for any vendor
    *  specific errors that occur.<br />
    */
   public long updateResource(String id, String commands) throws XMLDBException {
      XMLResource result = updateResourceResult(id, commands);
      return getResultCount(result);
   }
   
   protected long getResultCount(XMLResource resource) throws XMLDBException {
      Document doc = (Document) resource.getContentAsDOM();
      // Get the src:modified element
      Node node = doc.getDocumentElement().getFirstChild();
      // The count is a text node within the element.
      String count = node.getFirstChild().getNodeValue();
      
      try {
         return Long.parseLong(count);
      }
      catch (Exception e) {
         return 0;
      }
   }
}

