package org.apache.xindice.server;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: ResourceManager.java,v 1.1.1.1 2001/12/06 19:33:55 bradford Exp $
 */

import java.io.*;

/**
 * ResourceManager is an interface that is implemented to manage
 * shared resources. Implementations are responsible for managing these objects
 * in a custom-fashion. StdResourceManager is the standard
 * implementation of this interface.
 *
 * @see org.apache.xindice.server.standard.StdResourceManager
 */

public interface ResourceManager {
   /**
    * setResource places a resource by name into the
    * ResourceManager.  The resource will be placed into the name
    * space for the provided Versioning object.
    *
    * @param versioning the version space to use
    * @param name the resource name
    * @param object the object resource
    * @throws ParameterException if the object cannot be stored
    */
   void setResource(Versioning versioning, String name, Object object) throws ParameterException;

   /**
    * setResource places a resource by name into the
    * ResourceManager. The resource will be placed into the default
    * name space.
    *
    * @param name the resource name
    * @param object the object resource
    * @throws ParameterException if the object cannot be stored
    */
   void setResource(String name, Object object) throws ParameterException;

   /**
    * removeResource removes a resource by name from the
    * ResourceManager.  The resource will be removed from the name
    * space for the provided Versioning object.
    *
    * @param versioning the version space to use
    * @param name the resource name
    */
   void removeResource(Versioning versioning, String name);

   /**
    * removeResource removes a resource by name from the
    * ResourceManager.  The resource will be removed from the
    * default name space.
    *
    * @param name the resource name
    */
   void removeResource(String name);

   /**
    * getResource retrieves a resource by name from the
    * ResourceManager.  The resource will be retrived from the name
    * space for the provided Versioning object.
    *
    * @param versioning the version space to use
    * @param name the resource name
    * @return the object resource
    */
   Object getResource(Versioning versioning, String name);

   /**
    * getResource retrieves a resource by name from the
    * ResourceManager.  The resource will be retrived from the
    * default name space.
    *
    * @param name the resource name
    * @return the object resource
    */
   Object getResource(String name);

   /**
    * lockResource locks a resource by name in the
    * ResourceManager.  Locking protects the resource from being
    * overwritten or vulturized by the ResourceManager's dead object
    * collector.  The resource will be locked in the name space for the
    * provided Versioning object.
    *
    * @param versioning the version space to use
    * @param name the resource name
    */
   void lockResource(Versioning versioning, String name);

   /**
    * lockResource locks a resource by name in the
    * ResourceManager.  Locking protects the resource from being
    * overwritten or vulturized by the ResourceManager's dead object
    * collector.  The resource will be locked in the default name space.
    *
    * @param name the resource name
    */
   void lockResource(String name);

   /**
    * unlockResource unlocks a resource by name in the
    * ResourceManager.  The resource will be locked in the name space
    * for the provided Versioning object.
    *
    * @param versioning the version space to use
    * @param name the resource name
    */
   void unlockResource(Versioning versioning, String name);

   /**
    * unlockResource unlocks a resource by name in the
    * ResourceManager.  The resource will be locked in the default
    * name space.
    *
    * @param name the resource name
    */
   void unlockResource(String name);
}
