package org.apache.xindice.server.methods;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: PUT.java,v 1.1.1.1 2001/12/06 19:33:56 bradford Exp $
 */

import org.apache.xindice.server.*;
import org.apache.xindice.util.*;
import java.io.*;

/**
 * PUT probably needs serious fixing
 */

public final class PUT extends SimpleScript {
   private int chunkSize = 32768;

   public void setConfig(Configuration config) throws XindiceException {
      super.setConfig(config);
      chunkSize = config.getIntAttribute("chunksize");
   }

   private void sendErrorMessage(Gateway gateway, String pathinfo) {
      gateway.logMessage(LogManager.LOG_ERROR, "Could Not Create File: "+pathinfo);
      gateway.sendError(500, "Could Not Create File: "+pathinfo);
   }

   public void run(Gateway gateway) {
      String pathinfo = gateway.getScriptPath();
      if ( gateway.getUser() == null ) {
         // This scripts requires some form of authentication to be run
         gateway.logMessage(LogManager.LOG_ERROR, "Authorization Required To Create File: "+pathinfo);
         gateway.sendError(401, "Authorization Required To Create File: "+pathinfo);
         return;
      }
      boolean success = false;
      if ( pathinfo.endsWith("/") ) {
         FileResource path = gateway.getFileResource(pathinfo);
         success = path.mkdirs();
      }
      else {
         FileResource file = gateway.getFileResource(pathinfo);
         FileResource path = gateway.getFileResource(file.getPath());
         path.mkdirs();

         try {
            int count = 0;
            byte[] buffer = new byte[chunkSize];
            InputStream is = gateway.getInputStream();
            OutputStream os = file.getOutputStream();
            do {
               count = is.read(buffer);
               if ( count > 0 )
                  os.write(buffer, 0, count);
            }
            while ( count == chunkSize );
            os.close();
            is.close();
            success = true;
         }
         catch ( Exception e ) {
            success = false;
         }
      }
      if ( success ) {
         PrintStream out = new PrintStream(gateway.getOutputStream());
         gateway.setResponseHeader("Content-type", "text/html");
         gateway.setResponseHeader("Document-type", "xindice/filecreate/1.0");
         out.println("<HTML>\n<HEAD>\n<TITLE>File Created</TITLE>\n</HEAD>\n<BODY BGCOLOR=\"#FFFFFF\">\n<TABLE BORDER=\"0\">");
         out.println("<TR><TD VALIGN=\"TOP\" COLSPAN=\"2\"><FONT FACE=\"Arial\"><STRONG>New file created on server</STRONG></FONT></TD></TR>");
         out.println("<TR>\n<TD ALIGN=\"RIGHT\" VALIGN=\"TOP\" NOWRAP><FONT FACE=\"Arial\">Path to file:</FONT></TD>");
         out.println("<TD VALIGN=\"TOP\"><FONT FACE=\"Arial\"><XINDICE_PATHINFO>"+gateway.getRequestHeader("PATH_INFO")+"</XINDICE_PATHINFO></FONT></TD></TR>");
         out.println("</TABLE>\n</BODY>\n</HTML>");
         gateway.send(201);
      }
      else
         sendErrorMessage(gateway, pathinfo);
   }

   public int getAccessMethods() {
      return ACCESS_WRITE;
   }
}
