package org.apache.xindice.util;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: DirectoryClassLoader.java,v 1.1.1.1 2001/12/06 19:33:58 bradford Exp $
 */

import java.io.*;
import java.util.*;

/**
 * DirectoryClassLoader reads classes relative to the
 * constructor-provided base directory. This class is typically used for loading
 * classes from a directory that isn't on the CLASSPATH.
 */

public final class DirectoryClassLoader extends ClassLoader {
   private File directory = null;
   private Map cache = Collections.synchronizedMap(new HashMap());
   private Map dates = Collections.synchronizedMap(new HashMap());

   public DirectoryClassLoader(String directory) {
      this.directory = new File(directory);
   }

   private File getClassFile(String classname) {
      String filename = classname.replace('.', File.separatorChar)+".class";
      return new File(directory, filename);
   }

   public boolean isModified(String classname) {
      Long value = (Long)dates.get(classname);
      if ( value != null ) {
         try {
            File file = getClassFile(classname);
            return file.lastModified() != value.longValue();
         }
         catch ( Exception e ) {
            return true;
         }
      }
      return false;
   }

   public Class loadClass(String classname, boolean resolve) throws ClassNotFoundException {
      try {
         Class c = (Class)cache.get(classname);
         if ( c == null ) {
            try {
               File file = getClassFile(classname);
               FileInputStream fis = new FileInputStream(file);
               byte[] data = new byte[fis.available()];
               fis.read(data);
               fis.close();
               c = defineClass(classname, data, 0, data.length);
               dates.put(classname, new Long(file.lastModified()));
            }
            catch ( Throwable t ) {
               c = findSystemClass(classname);
            }
            cache.put(classname, c);
         }
         if ( resolve )
            resolveClass(c);
         return c;
      }
      catch ( Exception e ) {
         throw new ClassNotFoundException(e.toString());
      }
   }
}
