package org.apache.xindice.util;

/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * $Id: FileCache.java,v 1.1.1.1 2001/12/06 19:33:58 bradford Exp $
 */

import java.io.*;
import java.util.*;

/**
 * FileCache caches the content of files in memory.
 */

public class FileCache {
   private Map cache = new WeakHashMap(); // of FileCacheInfo

   public FileCache() {
   }

   public final boolean isInCache(File file) {
      return (cache.get(file) != null);
   }
   
   public final boolean isInCache(String name) {
      return (cache.get(new File(name)) != null);
   }
   
   public final boolean isModified(String name) {
      return isModified(new File(name));
   }

   public final boolean isModified(File file) {
      FileCacheInfo finfo = (FileCacheInfo)cache.get(file);
      return !file.exists()
          || finfo == null
          || (file.lastModified() != finfo.lastModified);
   }
   
   public final byte[] getFile(String name) {
      return getFile(new File(name));
   }
   
   public final byte[] getFile(File file) {
      boolean reload = true;
      FileCacheInfo finfo = (FileCacheInfo)cache.get(file);
      long lastmod = file.lastModified();
      if ( !file.exists() )
         return null;
      if ( finfo != null )
         reload = (lastmod != finfo.lastModified);
      if ( reload ) {
         try {
            FileInputStream fis = new FileInputStream(file);
            byte[] content = new byte[fis.available()];
            fis.read(content);
            fis.close();
            finfo = new FileCacheInfo(file, lastmod, content);
            cache.put(file, finfo);
            return content;
         }
         catch ( Exception e ) {
            return null;
         }
      }
      else
         return finfo.content;
   }

   /**
    * FileCacheInfo
    */
   private class FileCacheInfo {
      public File file;
      public long lastModified = 0;
      public byte[] content;

      public FileCacheInfo(File file, long lastModified, byte[] content) {
         this.file = file;
         this.lastModified = lastModified;
         this.content = content;
      }
   }
}
